#include <LCD3Wire.h>
#include <Servo.h>

const int pinDout = 10;
const int pinSTR = 12;
const int pinCLK = 11;
const int pinLevel = A1;
const int pinHand = A3;  //Moet analoge input worden, aangezien we een servomotor meer gaan gebruiken.
const int pinMotorThumbA = 3;
const int pinMotorA = 5;
const int pinMotorB = 6;
const int pinLeds[6] = {4, 2, 7, 1, 8, 0};
const int pinButtons = A0;
const int pinRandomSeed = A4;
const int pinOK = 13;
const int pinRepeat = A2;

const int lcdLines = 2;
const int Tolerance = 22;
const char Finger[5][7] = {"Thumb", "Index", "Middle", "Ring", "Pinky"};
const float calibrationDistance = 2.56;
int calibrationAngle;
int Hand;
int Level;
long button;
int count;
int exerciseRepeat;

//Used for determining number of digits of input number, used in LcdNumberWrite.
int estimateDigits(int nr) {
  int dec = 10;
  int temp = 1;
  int div = nr/dec;
  while(div > 0)
  {
    dec *= 10;
    div = nr/dec;
    temp++;
  }
  return temp;
}

//Adjusted function for printing numbers on the LCD screen, library function prints ASCII code according to the input number.
//LcdNumberWrite turns input number into its ASCII value, so the LCD screen prints the input number.
void LcdNumberWrite(int nr, LCD3Wire _lcd) {
  int dec;
  int div;
  int digits = estimateDigits(nr);
  for(int i = digits - 1; i >= 0; i--)
  {
    dec = pow(10, i);
    div = nr/dec;
    _lcd.print(div + 48);
    if(div > 0)
    {
      nr -= div*dec;
    }
  }
}

int readLevel(const int _pinLevel, const int _pinOK, LCD3Wire _lcd) {
  int Read;
  int lastLevel;
  int valueLevel;
  while(digitalRead(_pinOK) != HIGH)
  {
    Read = analogRead(_pinLevel);
    if((Read > 0) && (Read < 300) && (lastLevel != 1))
    {
      valueLevel = 1;
      _lcd.clear();
      _lcd.printIn("Level: ");
      LcdNumberWrite(valueLevel, _lcd);
    }
    else if((Read > 300) && (Read < 700) && (lastLevel != 2))
    {
      valueLevel = 2;
      _lcd.clear();
      _lcd.printIn("Level: ");
      LcdNumberWrite(valueLevel, _lcd);
    }
    else if((Read > 700) && (Read < 1024) && (lastLevel != 3))
    {
      valueLevel = 3;
      _lcd.clear();
      _lcd.printIn("Level: ");
      LcdNumberWrite(valueLevel, _lcd);
    }
    lastLevel = valueLevel;
  }
  return valueLevel;
}

char readHand(const int _pinHand, const int _pinOK, LCD3Wire _lcd) {
  int valueHand;
  int hand;
  while (digitalRead(_pinOK) != HIGH)
  {
    valueHand = analogRead(_pinHand);
    if(valueHand > 600) 
    {
      hand = 1;
      _lcd.cursorTo(2, 0);
      _lcd.printIn("Left hand ");
    }
    else 
    {
      hand = 0;
      _lcd.cursorTo(2, 0);
      _lcd.printIn("Right hand");
    }
  }
  return hand;
}

int readRepeat(const int _pinRepeat, const int _pinOK, LCD3Wire _lcd) {
  int Read;
  int Repeat;
  int Margin = 17;
  int posibleRepeats[19] = {17, 71, 125, 179, 233, 287, 341, 395, 449, 503, 557, 611, 665, 719, 773, 827, 881, 935, 989};
  _lcd.clear();
  _lcd.cursorTo(1, 0);
  _lcd.printIn("Exercise repeats");
  delay(2000);
  _lcd.clear();
  _lcd.cursorTo(1, 0);
  _lcd.printIn("Repeats: ");
  while (digitalRead(_pinOK) != HIGH)
  {
    Read = analogRead(_pinRepeat);    //Eventueel binnen for-lus zetten
    for (int i = 0; i <= 18; i++)
    {
      if ((Read > posibleRepeats[i] - Margin) && (Read < posibleRepeats[i] + Margin))
      {
        Repeat = 10 + i*5;
      }
    }
    _lcd.cursorTo(2, 0);
    LcdNumberWrite(Repeat, _lcd);
    _lcd.printIn("  ");
  }
  return Repeat;
}

float angleConvert(float _distance, int nr) {
  float Angle;
  switch(nr)
  {
    case 0: Angle = - 20.081*pow(_distance, 5) + 234.9*pow(_distance, 4) - 1122.2*pow(_distance, 3) + 2758.1*pow(_distance, 2) - 3542.9*_distance + 1945.9;
    break;
    case 1: Angle = 75.928*_distance - 110.3;
    break;
    case 2: Angle = 75.866*_distance - 104.52;
    break;
  }
  return Angle;
}

//Initialize motors according to chosen level, hand and number of repeats.
void motorInit(Servo _Motors[], int _Level) {
  float angleLevel;
  float Distance;
  if(_Level > 0)
  {
    switch(_Level)
    {
      case 1: 
      Distance = 3;
      angleLevel = angleConvert(Distance, 2);
      break;
      case 2:
      Distance = 3.5;
      angleLevel = angleConvert(Distance, 2);
      break;
      case 3:
      Distance = 4;
      angleLevel = angleConvert(Distance, 2);
      break;
    }
  }
  else
  {
    _Level = -_Level;    //Zo is _Level zeker positief voor de switch van de niet-duimvingers
    switch(_Level)
    {
      case 1: 
      Distance = 3;
      angleLevel = angleConvert(Distance, 2);
      break;
      case 2:
      Distance = 2.5;
      angleLevel = angleConvert(Distance, 2);
      break;
      case 3:
      Distance = 2;
      angleLevel = angleConvert(Distance, 2);
      break;
    }
  }
  _Motors[0].write(round(angleLevel));
  delay(15);
  switch(_Level)
  {
    case 1:
    Distance = 2;
    angleLevel = angleConvert(Distance, 2);
    break;
    case 2:
    Distance = 2.2;
    angleLevel = angleConvert(Distance, 2);
    break;
    case 3:
    Distance = 2.4;
    angleLevel = angleConvert(Distance, 2);
    break;
  }
  _Motors[1].write(round(angleLevel));
  _Motors[2].write(round(angleLevel));
  delay(15);
}

boolean checkButton(const int _button, const int _pinButtons, LCD3Wire _lcd, const int _tolerance) {
  boolean pushed = false;
  const unsigned int timeLimit = 5000;
  const unsigned int valueButtons[6] = {22, 218, 415, 614, 844, 922};
  unsigned int value;
  unsigned long currentMillis = millis();
  unsigned long previousMillis = 0;
  while(pushed == false)
  {
    value = analogRead(_pinButtons);
    if((value > valueButtons[_button] - _tolerance) && (value < valueButtons[_button] + _tolerance))
    {
      pushed = true;
    }
    if(currentMillis - previousMillis > timeLimit)
    {
      _lcd.cursorTo(2, 0);
      _lcd.printIn("You can do it!");
      previousMillis = currentMillis;
    }
  }
  return pushed;
}

LCD3Wire lcd = LCD3Wire(lcdLines, pinDout, pinSTR, pinCLK);
Servo Motors[3];

void setup() {
  delay(3000);
  lcd.init();
  delay(1000);
  lcd.printIn("Welcome!");
  
  Motors[0].attach(pinMotorThumbA);
  Motors[1].attach(pinMotorA);
  Motors[2].attach(pinMotorB);
  
  calibrationAngle = angleConvert(calibrationDistance, 2);
  
  Motors[0].write(round(calibrationAngle));
  Motors[1].write(round(calibrationAngle));
  Motors[2].write(round(calibrationAngle));
  
  for(int i = 0; i <= 5; i++)
  {
    pinMode(pinLeds[i], OUTPUT);
    digitalWrite(pinLeds[i], LOW);
    Serial.begin(9600);
  }
  
  pinMode(pinOK, INPUT);
  
  pinMode(9, OUTPUT);    //Enige niet gebruikte pin op OUTPUT zetten, wat hoge impedantie betekent en dus een veiligheid is
  
  randomSeed(analogRead(pinRandomSeed));
  
  delay(1000);
}

void loop() {
  lcd.clear();
  lcd.printIn("Choose level");
  delay(2000);
  Level = readLevel(pinLevel, pinOK, lcd);
  
  lcd.cursorTo(2,0);
  lcd.printIn("Choose hand");
  delay(2000);
  lcd.clear();
  lcd.printIn("Level: ");
  Serial.println(Level);
  lcd.print(Level + 48);
  lcd.cursorTo(2, 0);
  Hand = readHand(pinHand, pinOK, lcd);
  
  delay(2000);
  
  exerciseRepeat = readRepeat(A2, pinOK, lcd);
  
  delay(2000);
  
  lcd.clear();
  lcd.cursorTo(1, 0);
  lcd.printIn("Initializing...");
  
    delay(5000);
    
  if(Hand == 0)
  {
    motorInit(Motors, Level);
  }
  else if(Hand == 1)
  {
    Level = -Level; //motorInit uses the sign of Level to know which hand was chosen.
    motorInit(Motors, Level);
  }
  delay(1000);
  count = 0;
  lcd.clear();
  lcd.cursorTo(1,0);
  lcd.printIn("Count: ");
  LcdNumberWrite(count, lcd);
  while (count <= exerciseRepeat)
  {
    button = random(0, 5);
    if ((button == 0) && (Hand == 0))
    {
      button = 5;
    }
    digitalWrite(pinLeds[button], HIGH);
    if(checkButton(button, pinButtons, lcd, Tolerance))
    {
      count++;
      digitalWrite(pinLeds[button], LOW);
      lcd.clear();
      lcd.cursorTo(1, 0);
      lcd.printIn("Count: ");
      LcdNumberWrite(count, lcd);
    }
  }
  lcd.clear();
  lcd.printIn("Nice! Next exercise!");
  delay(2000);
}
